/***************************************************************************
    plugin.h
    -------------------
    begin                : Apr 20, 2009
    copyright            : (C) 2004 by Sergey Yakushev
    email                : yakushevs@list.ru

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*  $Id: plugin.h 9138 2009-04-20 10:17:31Z YEKST $ */
/***************************************************************************
 *   QGIS Programming conventions:
 *
 *   mVariableName - a class level member variable
 *   sVariableName - a static class level member variable
 *   variableName() - accessor for a class member (no 'get' in front of name)
 *   setVariableName() - mutator for a class member (prefix with 'set')
 *
 *   Additional useful conventions:
 *
 *   theVariableName - a method parameter (prefix with 'the')
 *   myVariableName - a locally declared variable within a method ('my' prefix)
 *
 *   DO: Use mixed case variable names - myVariableName
 *   DON'T: separate variable names using underscores: my_variable_name (NO!)
 *
 * **************************************************************************/
#ifndef VRPPLUGIN
#define VRPPLUGIN

#include "settings.h"
#include "utils.h"

//QT4 includes
#include <QObject>

//QGIS includes
#include <qgisplugin.h>
#include <qgspoint.h>

//forward declarations
class QAction;
class QToolBar;
class QPainter;
class QgisInterface;
class QDockWidget;

//forward declarations VRP plugins classes
class VrpPluginDirectionEditorMapTool;
class VrpPluginShortestPathWidget;

/**
* \class VrpPlugin
* \brief VRP plugin for QGIS
* This plugin solve the vehicle routing problem and etc...
*/
class VrpPlugin: public QObject, public QgisPlugin
{
	Q_OBJECT
public:

	/**
	 * Constructor for a plugin. The QgisInterface pointer is passed by
	 * QGIS when it attempts to instantiate the plugin.
	 * @param theInterface Pointer to the QgisInterface object.
	 */
	VrpPlugin( QgisInterface * theInterface );
	//! Destructor
	virtual ~VrpPlugin();
	/**
	* get adjacency matrix
	*/
	AdjacencyMatrix adjacencyMatrix( const QList< QgsPoint >& additionPoints);
	/**
	 * return pointer to my Interface
	 */
	QgisInterface *iface();
	/**
	 * return pointer to my Settings
	 */
	VrpPluginSettings* settings();
public slots:
    	void render(QPainter *painter);
	//! init the gui
   	virtual void initGui();
    	//!set values onthe gui when a project is read or the gui first loaded
    	virtual void projectRead();
	//!set default values for new project
    	void newProject();
	//! Show the property dialog box
    	void property(); 
    	//! unload the plugin
    	void unload();
    	//! show the help document
    	void help();

private slots:
	/**
	 * check toggle myLayer editing
	 */
	void onToggleEditing();
	/**
	 * set show roads direction and activate this editable
	 */
	void onShowDirection();
	/**
	 * show gui to find shortest path
	 */
	void onShortestPath();
	//! set direction editor mode
	void onEditDirection( bool );
    	//! 

private:
	/** 
	 * set all gui elements to default status
	 */
	void setGuiElementsToDefault( );
	/**
	 * draw the direction arrow for roads segment (x1,y1)-(x2,y2) with direction type dirType
	 */
	void renderArrow( QPainter * , double x1, double y1, double x2, double y2, VrpPluginSettings::DirectionType dirType);
	/**
	 * return distance from lenght <x1,y1><x2,y2> to point <x,y> or infinity() value
	 * in union point
	 */
	double distance(const QgsPoint& p1, const QgsPoint& p2, const QgsPoint& p, QgsPoint& center);

private:
    
    	////////////////////////////////////////////////////////////////////
	//
	// MANDATORY PLUGIN PROPERTY DECLARATIONS  .....
	// 
	////////////////////////////////////////////////////////////////////
	int mPluginType;
    	//! Pointer to the QGIS interface object
    	QgisInterface *mQGisIface;
    	//!pointer to the qaction for this plugin

	////////////////////////////////////////////////////////////////////
	// ADD YOUR OWN PROPERTY DECLARATIONS AFTER THIS POINT.....
    	//
    	////////////////////////////////////////////////////////////////////
	/**
	 * The plugin settings
	 */
	VrpPluginSettings mSettings;
	/**
	 * on show settings
	 */
    	QAction * mQSettingsAction;
	/**
	 * pointer ot the direction show action
	 */
	QAction * mQShowDirectionAction;
    	/**
	 * pointer to the direction editor action
	 */
    	QAction * mQEditDirectionAction;
	/**
	 * direction editor map tool
	 */
	VrpPluginDirectionEditorMapTool * mDirectionEditorMapTool;

	/**
	 * GUI for use shortest path finder
	 */
	VrpPluginShortestPathWidget *mQShortestPathDock;

private:
	static const int mArrowSize = 5;
};

#endif //VRPPLUGIN
