/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is a plugin generated from the QGIS plugin template              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/**
 * \file utils.cpp
 * \brief implementation of VRP plugins utils
 */

#include "utils.h"

//qt includes

// Qgis includes
#include "qgspoint.h"

// standart includes
#include <set>

double infinity()
{
	return 1e12;
}


bool QgsPointCompare::operator()(const QgsPoint& a, const QgsPoint& b) const
{
	return a.x() == b.x() ? a.y() < b.y() : a.x() < b.x();
}

AddonSort::AddonSort( const QgsPoint& p1, const QgsPoint& p2)
{
	double A = p1.y() - p2.y();
	double B = p2.x() - p1.x();
	double C = p1.x()*(-A) + p1.y()*(-B);
	mA = B;
	mB = -A;
	mC = -p1.x()*mA - p1.y()*mB;
}
bool AddonSort::operator() (const QgsPoint& a, const QgsPoint& b) const
{
	return (mA*a.x() +mB*a.y() + mC)<(mA*b.x() + mB*b.y() + mC);
}

ArcAttributes::ArcAttributes()
{
	mCost = infinity();
	mTime = infinity();
}
ArcAttributes::ArcAttributes(double cost, double time, int featureId) :
	mCost(cost), mTime(time), mFeatureId(featureId)
{
	
}
ArcPath::ArcPath(const QgsPoint& pt) :
	mCost(0), mTime(0)
{
	mPath.push_back( pt );
}
ArcPath::ArcPath()
{
	mCost = infinity();
	mTime = infinity();
}
double ArcPath::cost() const
{
	return mCost;
}
double ArcPath::time() const
{
	return mTime;
}
const QList< QgsPoint >& ArcPath::path() const
{
	return mPath;
}
void ArcPath::addPoint( const QgsPoint& p, double cost, double time)
{
	mPath.push_front( p );
	mCost += cost;
	mTime += time;
}

DijkstraFinder::DijkstraFinder( const AdjacencyMatrix& m, DijkstraFinder::OptimizationCriterion c ):
	mAdjacencyMatrix (m), mCriterion( c )
{
}

std::map< QgsPoint , DijkstraFinder::DijkstraIterator, QgsPointCompare> DijkstraFinder::find( const QgsPoint& p )
{
	CompareDijkstraIterator ci( mCriterion );
	std::set< DijkstraIterator, CompareDijkstraIterator > not_begin( ci );
	std::set< DijkstraIterator, CompareDijkstraIterator >::iterator it;
	std::map< QgsPoint, DijkstraIterator, QgsPointCompare> res;
	if (mAdjacencyMatrix.find( p ) == mAdjacencyMatrix.end() )
		return res;

	AdjacencyMatrixString::const_iterator arcIt;
	AdjacencyMatrixString::const_iterator end = mAdjacencyMatrix.find( p )->second.end();

	DijkstraIterator f;
	f.mCost = 0;
	f.mTime = 0;
	f.mFrontPoint = p;
	f.mBackPoint = p;
	res[ p ] = f;
	not_begin.insert( f );

	while ( !not_begin.empty() )
	{
		it = not_begin.begin();
		DijkstraIterator i = *it;
		not_begin.erase( it );

		if ( mAdjacencyMatrix.find( i.mBackPoint ) == mAdjacencyMatrix.end() )
		{
			continue;
		}
		end = mAdjacencyMatrix.find( i.mBackPoint )->second.end();
		for (arcIt = mAdjacencyMatrix.find( i.mBackPoint )->second.begin(); arcIt != end; ++arcIt)
		{
			DijkstraIterator di = i;
			di.mCost += arcIt->second.mCost;
			di.mTime += arcIt->second.mTime;

			if ( ci(di, res[ arcIt->first ] ) ) 
			{
				di.mFrontPoint = di.mBackPoint;
				di.mBackPoint = arcIt->first;
//				not_begin.erase( res[arc_it->first] );
				not_begin.insert( di );
				res[ arcIt->first ] = di;
			}
		}
	}
	return res;
}
ArcPath DijkstraFinder::find( const QgsPoint& frontPoint, const QgsPoint& backPoint )
{
	std::map< QgsPoint , DijkstraFinder::DijkstraIterator, QgsPointCompare> r = find( frontPoint );
	std::map< QgsPoint , DijkstraFinder::DijkstraIterator, QgsPointCompare>::iterator it;
	if ( r.find( backPoint ) == r.end() )
	{
		return ArcPath();

	}

	ArcPath path;
	QgsPoint nextPoint = backPoint;

	while (1)
	{
		path.mPath.push_front( nextPoint );
		if ( r[ nextPoint ].mFrontPoint == r[ nextPoint ].mBackPoint )
			break;
		nextPoint = r[ nextPoint ].mFrontPoint;
	}
	path.mCost = r[ backPoint ].mCost;
	path.mTime = r[ backPoint ].mTime;

	return path;
}

WorldMap DijkstraFinder::find( const QList< QgsPoint >& points )
{
	WorldMap worldMap;
	QList< QgsPoint >::const_iterator it1,it2;
	for (it1 = points.constBegin(); it1 != points.constEnd(); ++it1)
	{
		std::map< QgsPoint , DijkstraFinder::DijkstraIterator, QgsPointCompare> r = find( *it1 );
	        std::map< QgsPoint , DijkstraFinder::DijkstraIterator, QgsPointCompare>::iterator it;
		for (it2 = points.constBegin(); it2 != points.constEnd(); ++it2)
		{
		        if ( r.find( *it2 ) == r.end() )
		        {
	        	        worldMap[*it1][*it2] = ArcPath();
				continue;
		        }
			ArcPath path;
			QgsPoint nextPoint = *it2;

			while (1)
			{
				path.mPath.push_front( nextPoint );
				if ( r[ nextPoint ].mFrontPoint == r[ nextPoint ].mBackPoint )
					break;
				nextPoint = r[ nextPoint ].mFrontPoint;
			}
			path.mCost = r[ *it2 ].mCost;
			path.mTime = r[ *it2 ].mTime;
			
			worldMap[ *it1 ][ *it2 ] = path;
		}
	}
	return worldMap;
}
