/***************************************************************************
 *   Copyright (C) 2009 by Sergey Yakushev                                 *
 *   yakushevs@list.ru                                                     *
 *                                                                         *
 *   This is file define vrp plugins utils		                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef VRPPLUGIN_UTILS_H
#define VRPPLUGIN_UTILS_H

//#include <vrpguibase.h>
//  QT includes
#include <qlist.h>

// Qgis includes
#include "qgspoint.h"

// standart includes
#include <map>
#include <set>
// forward declaration Qgis-classes

/**
@author Sergey Yakushev
*/

/**
 * \function infinity()
 * \brief return big const double value
 */
double infinity();

/**
 * \class QgsPointCompare
 * \brief This class implement operation "<", ">", "==" and etc. for use QgsPoint type in STL containers
 *
 */

class QgsPointCompare
{
public:
	bool operator()(const QgsPoint& a, const QgsPoint& b) const;
};

/**
 * \class ArcAttributes
 * \brief This class contained arcs attributes.
 */
class ArcAttributes
{
public:
	ArcAttributes();
	ArcAttributes(double, double, int);
public:
	double mCost;
	double mTime;
	int mFeatureId;
};

typedef std::map< QgsPoint, ArcAttributes, QgsPointCompare> AdjacencyMatrixString;
typedef std::map< QgsPoint, AdjacencyMatrixString,  QgsPointCompare> AdjacencyMatrix;

typedef std::map< QgsPoint, std::list< QgsPoint > , QgsPointCompare> AdjacencyMatrixStringAddon;
typedef std::map< QgsPoint, AdjacencyMatrixStringAddon, QgsPointCompare> 
	AdjacencyMatrixAddon;;


/**
 * \class AddonSort
 * \brief implement sort compare for addons points
 */
class AddonSort 
{
public:
	AddonSort( const QgsPoint& p1, const QgsPoint& p2 );
	bool operator()(const QgsPoint& a, const QgsPoint& b) const;
private:
	double mA,mB,mC;
};
/**
 * \class ArcPath
 * \brief This class contained path arcs and path attributes
 */
class ArcPath
{
public:
	/**
	 * warning: mCost and mTime is the infinity by default
	 */
	ArcPath();
	/**
	 * This path contained one point and zeros cost and time/
	 */
	explicit ArcPath( const QgsPoint& pt);
	/**
	 * \return cost of path
	 */
	double cost() const;
	/**
	 * \return time of path
	 */
	double time() const;
	/**
	 * \return path
	 */
	const QList< QgsPoint >& path() const;
	/**
	 * addPoint to start of path and increment mCost and mTime
	 */
	void addPoint( const QgsPoint& pt, double cost, double time);

public:
	double mCost;
	double mTime;
	QList< QgsPoint > mPath;
};

typedef std::map< QgsPoint, ArcPath, QgsPointCompare > WorldMapString;
typedef std::map< QgsPoint, WorldMapString, QgsPointCompare > WorldMap;



/**
 * \class DijkstraFinder
 * \brief This class find shortest path via two points using Dijkstra's algorithm
 */
class DijkstraFinder 
{
public:
	enum OptimizationCriterion { byTime=1, byCost=2 };
private:
	class DijkstraIterator {
	public:
		DijkstraIterator()
		{
			mCost = infinity();
			mTime = infinity();
		}
		double mCost;
		double mTime;
		QgsPoint mBackPoint;
		QgsPoint mFrontPoint;
	};
	class CompareDijkstraIterator {
	public:
		CompareDijkstraIterator( OptimizationCriterion criterion = byCost ) :
			mCriterion( criterion )
		{ }
		bool operator()( const DijkstraIterator& a, const DijkstraIterator& b ) const
		{
			if ( mCriterion == byCost )
			{
				return a.mCost < b.mCost;
			}
			return a.mTime < b.mTime;
		}
		bool operator==( const CompareDijkstraIterator& a ) const
		{
			return mCriterion == a.mCriterion;
		}
	private:
		OptimizationCriterion mCriterion;
	};
public:
	/**
	 * constructor.
	 * m is adjacency matrix of graph, criterion is a citerion of shortest path
	 */
	DijkstraFinder( const AdjacencyMatrix &m, OptimizationCriterion criterion );

	/**
	 * find all shortest path from point frontPoint to all points of graph.
	 * return tree.
	 */
	std::map< QgsPoint , DijkstraIterator, QgsPointCompare > find( const QgsPoint& p );
	/**
	 * return shortest path form point frontPoint to endPoint
	 */
	ArcPath find( const QgsPoint& frontPoint, const QgsPoint& endPoint );
	/**
	 * return Matrix of Paths, times and costs of shortest path from points
	 */
	WorldMap find( const QList< QgsPoint>& points );
private:
	const AdjacencyMatrix& mAdjacencyMatrix;
	OptimizationCriterion mCriterion;
};
#endif
